/*
 *  Classified Tydeman Consulting Proprietary
 *  Copyright (C) 1996, 2020 Tydeman Consulting.  All rights reserved.
 *
 *  Fred J. Tydeman	tydeman@tybor.com
 *
 * - -- --- ---- ----- -------------------------------------------------
 *
 *    This code is copyrighted.  Under the copyright laws, this code may
 *	   not be copied, in whole or part, without prior written
 *	   consent of Tydeman Consulting.
 *
 *    RESTRICTED RIGHTS LEGEND: Use, duplication, or disclosure by the
 *	   government is subject to restrictions as set forth in
 *	   subparagraph (c)(1)(ii) of the Rights in Technical Data and
 *	   Computer Software clause at DFARS 252.227-7013 and
 *	   FAR 52.227-19.
 *
 * - -- --- ---- ----- -------------------------------------------------
 * $Revision: 2.1 $
 ***********************************************************************
 *  FILE: g1150000.h
 *
 *  Generic "Macro" included by t1150000.c, t1151000.c, t1152000.c - IEEE
 *  Generic "Macro" included by t1150001.c, t1151001.c, t1152001.c - general
 *  This is the real code for testing sin/sinf/sinl function.
 *
 *  Assertion being tested:
 *
 * 4.3.2.6 sin
 *
 * p. 40, l. 8: sin is sine of radians
 * {t1150000.c}  Test with ...
 * Invalid operands (domain errors)
 *	f(+/- INFINITY) is NaN and raises invalid
 * Exact unexception values
 *	f(NaN) is the same NaN and raises no exceptions
 *	f(+/- 0.0) is +/- 0.0
 * Exact exceptional values
 *	None
 * Inexact exceptional values
 *	f( denorm ) is same denorm and raises underflow and inexact
 *	None for large arguments (but LIA-2 discusses Arg-too-big exception)
 * Accuracy
 *	1.5*rnd_error while |x| < max_arg_sin from LIA-2
 *	2.0 ulp while |x| < radix**(mantissa_bits/2) from ADA
 *	This can be tested in two ways:  the basic cycle of 0 ... 2 PI, and
 *	argument reduction with values determined by continued fraction
 *	representation of PI.
 * Symmetries
 *	f(x) = -f( -x ) for finite arguments
 * Other properties
 *	-1.0 <= f(x) <= +1.0, for finite arguments
 *	sin(x)**2 + cos(x)**2 = 1 +/- ? ulps, for finite arguments
 *	Result MAY depend upon the current rounding direction.
 * Raise (only deserved) exceptions
 * Not clear the flags of the caller
 * Not alter the modes of the caller
 *
 * Notes:
 *	Bugs caught by this test so far (and how to bypass):
 *	f() missing or very buggy		FIX_FUNCS_sin
 *	nan*() functions missing		FIX_FUNCS_nan
 *	Denorm constants flushed to zero	FIX_DEC2BIN_ZERO
 *	Denormals converted wrong		FIX_DENORM_TWO
 *	INF converted to pseudo INF => NaN	FIX_PSEUDO_INFINITY
 *	f(x) not accurate enough		ULP_sin
 *	f(-x) is not -f(x)			FIX_EVENODD_sin
 *	Too few or many flags raised		FIX_FLAG_sin
 *	f() clears some flags			FIX_CLR_FLAGS_sin
 *	f() depends upon flags			FIX_DEPENDS_FLAGS_sin
 *	f() does inexact wrong			FIX_INEXACT_sin
 *	f(denorm) not raise underflow		IMPL_UNDERFLOW_DEN_LOSS
 *	f(NaN) is not quiet			FIX_F_OF_NAN_IS_QUIET
 *	f(NaN) is not NaN			FIX_F_OF_NAN_IS_NAN
 *	f(NaN) is not same NaN			IMPL_DIFF_NAN
 *	f(-0.0) is not same -0.0		FIX_SAME_ZERO
 *	f(NaN) is wrong				FIX_NAN_sin
 *	f(zero) is wrong			FIX_ZERO_sin
 *	f(denorm) is wrong			FIX_DEN_sin
 *	f(small) is wrong			FIX_SMALL_sin
 *	f(large) is wrong			FIX_LARGE_sin
 *	f(INF) is wrong				FIX_INF_sin
 *	f() is not NaN				FIX_NAN_RES_sin
 *	f() is not zero				FIX_ZERO_RES_sin
 *	f() is not denorm			FIX_DEN_RES_sin
 *	f() is not small			FIX_SMALL_RES_sin
 *	f() is not large			FIX_LARGE_RES_sin
 *	f() is not INF				FIX_INF_RES_sin
 *	f() is outside range			FIX_RANGE_sin
 *	(f)() not match f()			FIX_OVERLOAD_sin
 *	f() is NaN				FIX_NAN_GOT_sin
 *	f(arg) is arg				FIX_ARG_GOT_sin
 *	f(x) is zero [and f(-x) is not zero]	FIX_ZERO_GOT_sin
 *	f() alter FPU control modes		FIX_MODES_sin
 *	errno not consistent w/ flags		FIX_ERRNO_MATH_sin
 *	underflow: errno not consist. w/ flags	IMPL_UF_WEIRD_sin
 *
 *  MODIFICATION HISTORY:
 *
 * 1996/01/10 - Create skeleton from test plan document.
 * 1996/03/14 - Add more ways to fail.  TEST_SNAN.
 * 1996/03/17 - Suppress duplicate output.
 * 1996/03/25 - Remove spurious inexact for LD using D func
 * 1996/06/08 - Allow for unexpected NaNs; wrong sizeof info.
 * 1996/06/17 - Setup for general (non-IEEE) floating-point.
 * 1996/07/15 - Upgrade to C9X proposal WG14/N546.
 * 1996/08/11 - Add FIX_INEXACT_*.
 * 1996/10/01 - Allow for limited precision dropping NaN fract.
 * 1996/11/16 - Add FIX_FUNCS_*.
 * 1997/01/17 - Add FPCE_SAMPLE.
 * 1997/01/26 - *_t -> *_q (reserved by POSIX.1).
 * 1997/02/23 - FE_IEEE_EXCEPT -> FE_ALL_EXCEPT.
 * 1997/09/17 - Update email address.
 * 1997/10/03 - Only check required flags in flags3 == flags1.
 * 1997/10/04 - FIX_DEN_UNDER -> IMPL_UNDERFLOW_DEN_LOSS.
 * 1998/06/06 - Add OVERRIDE_TV.
 * 1998/07/11 - Add FIX_FE_FLAGS.
 * 1998/07/25 - Add FIX STATIC CONST.
 * 1998/07/31 - test(FE_ALL_EXCEPT) -> test(FE_IEEE_EXCEPT)
 * 1998/08/02 - Add FIX_FLAGS_feraiseexcept.
 * 1998/08/04 - FIX_SAME_ZERO -> LMS_SIGNED_ZERO_BUGS.
 * 1998/09/10 - Add FIX_NAN_IN_ARRAY and FIX_INF_IN_ARRAY.
 * 1998/09/13 - FIX STATIC CONST -> FIX_STATIC_VECTORS.
 * 1999/03/03 - Add LARGE_ARG_f and ULP_LARGE_ARG_f.
 * 1999/05/17 - Add FIX_NAN_GOT_sin.
 * 1999/05/20 - Add FIX_ZERO_GOT_sin.
 * 2000/03/01 - Make classification explicit in each file.
 * 2000/03/01 - Remove confidential so can post to public ftp site.
 * 2001/06/24 - Add FIX_ARG_GOT_sin.
 * 2003/05/28 - Add tests of type generic versus type specific function.
 * 2004/04/23 - Add uses of CLEAR_ALL and RAISE_ALL.
 * 2004/04/23 - Update DEBUG section.
 * 2004/07/18 - Add FIX_CONST_EXP_IN_INIT.
 * 2004/07/18 - Add ckFailed usage.
 * 2006/04/01 - Add VOLATILE for den calculation.
 * 2006/04/01 - Move SPECIFIC definition from g1* to t1*.
 * 2006/04/01 - Add support for Decimal FP.
 * 2006/04/01 - Add errno checking.
 * 2006/04/01 - Break into functions.
 * 2006/04/18 - Redo min and den section.
 * 2006/05/01 - Add g1rnd1.h.
 * 2006/06/02 - Lint cleanup.
 * 2006/06/21 - Redo g1errno.h and usage.
 * 2006/06/25 - Add return if ULP is zero.
 * 2006/07/17 - Add tests of spurious underflow.
 * 2006/07/17 - Update DEBUG section.
 * 2006/07/17 - Indent #if's.
 * 2006/08/26 - DECFP_OVERLOAD -> OVERLOAD10.
 * 2006/08/26 - Add print_val_q().
 * 2007/06/17 - ulp_q -> ulp_diff_q.
 * 2007/06/18 - min_q-den_q -> minm1_q.
 * 2007/06/18 - Add INT_ERR_f.
 * 2007/11/09 - Append uL to large int const.
 * 2007/11/27 - Allow for 6 digit INT_ERR_f.
 * 2008/02/12 - Add FP_RADIX items.
 * 2008/02/18 - Indent #if's.
 * 2008/04/13 - Update DEBUG section.
 * 2008/06/19 - Bypass tests when inf == arg and fake DFP on BFP.
 * 2008/06/30 - Add FIX_ARG_GOT_sin.
 * 2008/07/14 - Shorten marco symbols to less than 32 chars.
 * 2008/08/27 - Change MAX_q/TWO_q to MAX_q/RX.
 * 2008/08/28 - Add IMPL_COHORT_ZERO.
 * 2008/09/10 - Add nancmp_q() and ulp of SIGN_NAN.
 * 2008/09/28 - Update DEBUG section.
 * 2008/12/21 - Replace use of FMT-DEC with PRINT_VAL.
 * 2009/05/06 - Add FIX_ULI_FROM_LARGE_FP_CONST.
 * 2010/06/24 - Redo FIX_NAN_IN_ARRAY.
 * 2010/06/24 - Redo FIX_INF_IN_ARRAY.
 * 2010/06/30 - Redo chk_rnd and when is inexact.
 * 2010/06/30 - NAN_q -> NAN_Q.
 * 2010/06/30 - INF_q -> INF_Q.
 * 2010/07/21 - IMPL_SAME_NAN->IMPL_SIGN_NAN; FIX_SAME_NAN->IMPL_DIFF_NAN.
 * 2010/08/29 - Add FIX_STATIC_INF, FIX_STATIC_NAN.
 * 2011/01/06 - FIX_*_q -> LMS_*_q; IMPL_*_q -> LCL_*_q.
 * 2011/01/18 - Add usage of ASSIGN2( ).
 * 2012/04/01 - Add tests of more and more digits in range 0.5 <= x < 1.
 * 2012/04/01 - Add: if( NumFail >> 6 ) break;
 * 2012/04/01 - Add max_ulp_err.
 * 2012/04/11 - vd[ui].ulp==K  =>  K==vd[ui].ulp
 * 2012/05/03 - Add get_bfp_rnd(), get_dfp_rnd().
 * 2013/02/28 - Add FIX_DFP_FLAGS.
 * 2013/03/03 - Add IMPL_INEXACT10_*.
 * 2013/05/12 - Fix spelling of weird.
 * 2014/06/12 - Add TRIGGER for nan_q().
 * 2014/07/24 - FLUSH_DENORM -> FLUSH_RUN_DEN_BOTH.
 * 2014/07/28 - Add dbldblcmp().
 * 2014/08/01 - Redo tests for infinity due to DBL_DBL.
 * 2014/08/02 - Indent #if's.
 * 2014/08/03 - Add LMS_NAN_Q_IS_ZERO, LMS_INF_Q_IS_ZERO, LMS_USING_FP_EXP.
 * 2014/08/04 - Move NAN_Q, INF_Q, FP_EXP() to vpatch.h.
 * 2014/08/04 - Add LMS_MIN_Q_IS_ZERO.
 * 2014/08/05 - Add include of vpatch.
 * 2014/08/05 - Add LMS_MINp1_Q_IS_ZERO.
 * 2014/08/07 - Remove extra "=" from PRINT_VAL.
 * 2014/08/17 - Add ulp_fun (since literal may differ from memory).
 * 2014/08/18 - ULP_q -> EPS_q  and  ULP_f -> ERR_f
 * 2014/08/21 - Replace symbols with vars past end of cleanup().
 * 2014/08/29 - Add ck() to chk_spur_under().
 * 2014/09/13 - nancmp -> nosigncmp.
 * 2014/09/13 - ULP_* -> ERR_*.
 * 2015/11/16 - Allow for FIX_CLOSE_MIN_*.
 * 2020/04/21 - Move VOLATILE to mydefs.h.
 * 2020/09/06 - Redo LIMIT_DOMAIN.
 * 2020/09/06 - Update DEBUG section.
 * 2020/09/06 - Update PRTDEBUG.
 *
 ***********************************************************************/

/*
 * Local override of FIXes to force errors to show
 */
#if (defined DEBUG) && (!defined FPCE_SAMPLE)
  #ifdef DEBUG2	/* Kills some compilers */
    #undef  FIX_DFP_FLAGS
    #undef  FIX_FE_FLAGS
    #undef  FIX_FUNCS10_nan
    #undef  FIX_FUNCS10_sin
    #undef  FIX_FUNCS_nan
    #undef  FIX_FUNCS_sin
    #undef  FIX_OVERLOAD
    #undef  FIX_OVERLOAD10
    #undef  FIX_OVERLOAD10_sin
    #undef  FIX_OVERLOAD_sin
    #undef  FIX_STATIC_VECTORS
    #undef  FIX_ULI_FROM_LARGE_FP_CONST
    #undef  FIX_VOLATILE
    #undef  LIMIT_DOMAIN
    #undef  LMS_DEN_Q_IS_ZERO
    #undef  LMS_INF_Q_IS_ZERO
    #undef  LMS_MINm1_Q_IS_ZERO
    #undef  LMS_MINp1_Q_IS_ZERO
    #undef  LMS_MIN_Q_IS_ZERO
    #undef  LMS_mZERO
    #undef  LMS_NAN_Q_IS_ZERO
    #undef  LMS_NEED_FIXES_3_28
    #undef  LMS_ONEm1_Q_IS_ZERO
    #undef  LMS_ONEp1_Q_IS_ZERO
    #undef  LMS_SIGNED_ZERO_BUGS
    #undef  LMS_USING_FP_EXP
    #undef  NEED_VOLATILE
  #endif
#endif

#ifdef DEBUG	/* local override of FIXes to force errors to show */
  #ifdef DEBUG3
    #undef  IMPL_COHORT_ZERO
    #undef  IMPL_INEXACT10_sin
    #undef  IMPL_NO_ERRNO_RND10_sin
    #undef  IMPL_NO_ERRNO_RND_sin
    #undef  IMPL_NO_STRONG_RND10_sin
    #undef  IMPL_NO_STRONG_RND_sin
    #undef  IMPL_NO_WEAK_RND10_sin
    #undef  IMPL_NO_WEAK_RND_sin
    #undef  IMPL_UF_WEIRD10_sin
    #undef  IMPL_UF_WEIRD_sin
/*  #undef  DFP_TEST_LVL */
/*  #undef  FIX_BAD_FAKE_DECFP */
/*  #undef  FPCE_TEST_LVL */
  #endif
  #undef  DEN_FLAGS
  #define DEN_FLAGS (FE_UNDERFLOW|FE_INEXACT)	/* IMPL_UNDERFLOW_DEN_LOSS */
  #undef  FIX_ARG_GOT10_sin
  #undef  FIX_ARG_GOT_sin
  #undef  FIX_BAD_UF_GOT10_sin
  #undef  FIX_BAD_UF_GOT_sin
  #undef  FIX_CLOSE_MIN_DBL
  #undef  FIX_CLOSE_MIN_FLT
  #undef  FIX_CLOSE_MIN_LDBL
  #undef  FIX_CLR_FLAGS10_sin
  #undef  FIX_CLR_FLAGS_sin
  #undef  FIX_DEN10_sin
  #undef  FIX_DEN_RES10_sin
  #undef  FIX_DEN_RES_sin
  #undef  FIX_DEN_sin
  #undef  FIX_DEPENDS_FLAGS10_sin
  #undef  FIX_DEPENDS_FLAGS_sin
  #undef  FIX_ERRNO_MATH
  #undef  FIX_ERRNO_MATH10_sin
  #undef  FIX_ERRNO_MATH_sin
  #undef  FIX_EVENODD10_sin
  #undef  FIX_EVENODD_sin
  #undef  FIX_FLAG10_sin
  #undef  FIX_FLAGS_feraiseexcept
  #undef  FIX_FLAG_sin
  #undef  FIX_FLUSH_RUN_DEN_BOTH
  #undef  FIX_F_OF_NAN_IS_NAN
  #undef  FIX_INEXACT10_sin
  #undef  FIX_INEXACT_sin
  #undef  FIX_INF10_sin
  #undef  FIX_INF_RES10_sin
  #undef  FIX_INF_RES_sin
  #undef  FIX_INF_sin
  #undef  FIX_LARGE10_sin
  #undef  FIX_LARGE_RES10_sin
  #undef  FIX_LARGE_RES_sin
  #undef  FIX_LARGE_sin
  #undef  FIX_MODES10_sin
  #undef  FIX_MODES_sin
  #undef  FIX_NAN10_sin
  #undef  FIX_NAN_GOT10_sin
  #undef  FIX_NAN_GOT_sin
  #undef  FIX_NAN_RES10_sin
  #undef  FIX_NAN_RES_sin
  #undef  FIX_NAN_sin
  #undef  FIX_RANGE10_sin
  #undef  FIX_RANGE_sin
  #undef  FIX_SMALL10_sin
  #undef  FIX_SMALL_RES10_sin
  #undef  FIX_SMALL_RES_sin
  #undef  FIX_SMALL_sin
  #undef  FIX_ZERO10_sin
  #undef  FIX_ZERO_GOT_sin
  #undef  FIX_ZERO_RES10_sin
  #undef  FIX_ZERO_RES_sin
  #undef  FIX_ZERO_sin
  #undef  LARGE_ARG_f
  #define LARGE_ARG_f MAX_q
  #undef  NAN_FLAGS
  #define NAN_FLAGS 0		/* FIX_F_OF_NAN_IS_QUIET */
  #undef  ERR_M0
  #define ERR_M0 BIT4BIT	/* FIX_SAME_ZERO */
  #undef  ERR_NAN
  #define ERR_NAN BIT4BIT	/* IMPL_DIFF_NAN */
  #undef  ULP_sin
#endif

#ifndef   ULP_sin
  #define ULP_sin 1.0
#endif

#if 2 == FP_RADIX

  #define get_true_rnd() get_bfp_rnd()

  #ifdef    IMPL_UF_WEIRD_sin
    #define LCL_UF_WEIRD_q
  #else
    #undef  LCL_UF_WEIRD_q
  #endif

  #ifdef    FIX_FLAG_sin
    #define LMS_FLAG_q
  #else
    #undef  LMS_FLAG_q
  #endif

  #ifdef    FIX_MODES_sin
    #define LMS_MODES_q
  #else
    #undef  LMS_MODES_q
  #endif

  #ifdef    IMPL_NO_ERRNO_RND_sin
    #define LCL_NO_ERRNO_RND_q
  #else
    #undef  LCL_NO_ERRNO_RND_q
  #endif

  #ifdef    IMPL_NO_WEAK_RND_sin
    #define LCL_NO_WEAK_RND_q
  #else
    #undef  LCL_NO_WEAK_RND_q
  #endif

  #ifdef    IMPL_NO_STRONG_RND_sin
    #define LCL_NO_STRONG_RND_q
  #else
    #undef  LCL_NO_STRONG_RND_q
  #endif

#elif 10 == FP_RADIX

  #undef    IMPL_INEXACT_sin
  #ifdef    IMPL_INEXACT10_sin
    #define IMPL_INEXACT_sin IMPL_INEXACT10_sin
  #else
    #define IMPL_INEXACT_sin 0
  #endif

  #define get_true_rnd() get_dfp_rnd()

  #undef    FIX_FUNCS_sin
  #ifdef    FIX_FUNCS10_sin
    #define FIX_FUNCS_sin
  #endif

  #undef    FIX_FUNCS_nan
  #ifdef    FIX_FUNCS10_nan
    #define FIX_FUNCS_nan
  #endif

  #ifdef    IMPL_UF_WEIRD10_sin
    #define LCL_UF_WEIRD_q
  #else
    #undef  LCL_UF_WEIRD_q
  #endif

  #undef    FIX_FE_FLAGS
  #ifdef    FIX_DFP_FLAGS
    #define FIX_FE_FLAGS
  #endif

  #ifdef    FIX_FLAG10_sin
    #define LMS_FLAG_q
  #else
    #undef  LMS_FLAG_q
  #endif

  #ifdef    FIX_MODES10_sin
    #define LMS_MODES_q
  #else
    #undef  LMS_MODES_q
  #endif

  #ifdef    IMPL_NO_ERRNO_RND10_sin
    #define LCL_NO_ERRNO_RND_q
  #else
    #undef  LCL_NO_ERRNO_RND_q
  #endif

  #ifdef    IMPL_NO_WEAK_RND10_sin
    #define LCL_NO_WEAK_RND_q
  #else
    #undef  LCL_NO_WEAK_RND_q
  #endif

  #ifdef    IMPL_NO_STRONG_RND10_sin
    #define LCL_NO_STRONG_RND_q
  #else
    #undef  LCL_NO_STRONG_RND_q
  #endif

  #undef    FIX_ERRNO_MATH_sin
  #ifdef    FIX_ERRNO_MATH10_sin
    #define FIX_ERRNO_MATH_sin
  #endif

  #undef    FIX_NAN_sin
  #ifdef    FIX_NAN10_sin
    #define FIX_NAN_sin
  #endif

  #undef    FIX_ZERO_sin
  #ifdef    FIX_ZERO10_sin
    #define FIX_ZERO_sin
  #endif

  #undef    FIX_DEN_sin
  #ifdef    FIX_DEN10_sin
    #define FIX_DEN_sin
  #endif

  #undef    FIX_SMALL_sin
  #ifdef    FIX_SMALL10_sin
    #define FIX_SMALL_sin
  #endif

  #undef    FIX_LARGE_sin
  #ifdef    FIX_LARGE10_sin
    #define FIX_LARGE_sin
  #endif

  #undef    FIX_INF_sin
  #ifdef    FIX_INF10_sin
    #define FIX_INF_sin
  #endif

  #undef    FIX_NAN_RES_sin
  #ifdef    FIX_NAN_RES10_sin
    #define FIX_NAN_RES_sin
  #endif

  #undef    FIX_NAN_GOT_sin
  #ifdef    FIX_NAN_GOT10_sin
    #define FIX_NAN_GOT_sin
  #endif

  #undef    FIX_ZERO_RES_sin
  #ifdef    FIX_ZERO_RES10_sin
    #define FIX_ZERO_RES_sin
  #endif

  #undef    FIX_DEN_RES_sin
  #ifdef    FIX_DEN_RES10_sin
    #define FIX_DEN_RES_sin
  #endif

  #undef    FIX_SMALL_RES_sin
  #ifdef    FIX_SMALL_RES10_sin
    #define FIX_SMALL_RES_sin
  #endif

  #undef    FIX_LARGE_RES_sin
  #ifdef    FIX_LARGE_RES10_sin
    #define FIX_LARGE_RES_sin
  #endif

  #undef    FIX_INF_RES_sin
  #ifdef    FIX_INF_RES10_sin
    #define FIX_INF_RES_sin
  #endif

  #undef    FIX_INEXACT_sin
  #ifdef    FIX_INEXACT10_sin
    #define FIX_INEXACT_sin
  #endif

  #undef    FIX_RANGE_sin
  #ifdef    FIX_RANGE10_sin
    #define FIX_RANGE_sin
  #endif

  #undef    FIX_EVENODD_sin
  #ifdef    FIX_EVENODD10_sin
    #define FIX_EVENODD_sin
  #endif

  #undef    FIX_CLR_FLAGS_sin
  #ifdef    FIX_CLR_FLAGS10_sin
    #define FIX_CLR_FLAGS_sin
  #endif

  #undef    FIX_DEPENDS_FLAGS_sin
  #ifdef    FIX_DEPENDS_FLAGS10_sin
    #define FIX_DEPENDS_FLAGS_sin
  #endif

  #undef    FIX_BAD_UF_GOT_sin
  #ifdef    FIX_BAD_UF_GOT10_sin
    #define FIX_BAD_UF_GOT_sin
  #endif

  #undef    FIX_ARG_GOT_sin
  #ifdef    FIX_ARG_GOT10_sin
    #define FIX_ARG_GOT_sin
  #endif

#else
  #error Contact Tydeman
#endif

#if defined(NEED_VOLATILE) && (!defined FIX_VOLATILE)
#else
#endif

#define PRINT_VAL(s,x) print_val_q(s,x)

#if (10 == FP_RADIX)
  #define D_FP D10
  #define L_FP LD10
  #define PRINT_VALL(s,x) prt_ld10(s,x)
#else
  #define D_FP DBL
  #define L_FP LDBL
  #define PRINT_VALL(s,x) prt_ldbl(s,x)
#endif

#define PRINT_ARGS \
	if( !printed_arg ){\
	  printed_arg = 1;\
	  PRINT_VAL(" Input arg", vd[ui].arg );\
	}

#include "vpatch.h"	/* FP_EXP(), NAN_Q, INF_Q, DEN_Q, MINm1_Q, ... SIZE() */

static char hbuf1[HEX];
static int printed_arg;

typedef struct{
	FP arg;			/* input argument */
	FP res;			/* expected result */
	FP ulp;			/* allowed error, units last place */
	int flags_req;		/* required flags */
	int flags_opt;		/* optional flags */
	} vector;

/*lint -esym(613,vd) // Possible use of NULL pointer */
/*lint -esym(818,vd) -esym(952,vd) // Parm could be 'const' */

#if (!defined FIX_ERRNO_MATH) && (!defined FIX_ERRNO_MATH_sin)
  #include "g1errno.h"		/* chk_errno_flags(errno1, flags1, vd, ui) */
#else
  #ifndef   LCL_NO_ERRNO_RND_q
    #define LCL_NO_ERRNO_RND_q	/* Do not have chk_rnd look at errno */
  #endif
#endif

#if (!defined FIX_FUNCS_sin)
  #include "g1rnd1.h"		/* chk_rnd(FP, ulp, vd, ui, __LINE__) */
#endif

/*================================================================*/
/*
 * ONEp1_q == symbol used in t1*.fjt == ONEp1_FLT, ONEp1_DBL, ONEp1_LDBL
 * ONEp1_Q == symbol used in v1*.fjt; ==   PATCH, ONEp1_q == defined in vpatch.h
 * onep1_q == symbol or var used here == onep1_q, ONEp1_q
 */
#if (defined LMS_ONEp1_Q_IS_ZERO)	/* watcom 1.0 */
static VOLATILE FP onep1_q;
#else
  #define onep1_q ONEp1_q
#endif

#if (defined LMS_ONEm1_Q_IS_ZERO)
static VOLATILE FP onem1_q;
#else
  #define onem1_q ONEm1_q
#endif

static VOLATILE FP min_q = MIN_q;

/*================================================================*/
static void cleanup(vector vd[], const int testing_ieee, const int need_fixes_3_28){
  VOLATILE FP den_q;

#if (defined LMS_ONEp1_Q_IS_ZERO)	/* watcom 1.0 */
  onep1_q = EPS_q;
  onep1_q = ONE_q + onep1_q;
#endif

#if (defined LMS_ONEm1_Q_IS_ZERO)
  onem1_q = EPS_q;
  #if (defined FIX_BAD_FAKE_DECFP) && (10==FP_RADIX)
  onem1_q = onem1_q / TWO_q;
  #else
  onem1_q = onem1_q / RX;
  #endif
  onem1_q = ONE_q - onem1_q;
#endif

#if (!defined FIX_FLUSH_RUN_DEN_BOTH)
  den_q = min_q * EPS_q;
  min_q = den_q / EPS_q;		/* in case MIN_q is wrong	*/
  #if (defined LMS_MIN_Q_IS_ZERO)
  ck( ZERO_q != min_q );
  if( ZERO_q == min_q ) min_q = MIN_q;	/* MIN_q is too small */
  #else
    #if (defined FIX_CLOSE_MIN_FLT) || (defined FIX_CLOSE_MIN_DBL) || (defined FIX_CLOSE_MIN_LDBL)
  { VOLATILE FP min_qq = MIN_q;
    ck( min_q == min_qq );
  }
    #else
  ck( min_q == MIN_q );
    #endif
  #endif
#endif

  if( testing_ieee ){

#ifdef TESTING_IEEE_VECTORS

    /*
     * If NAN_FLT, NAN_DBL, or NAN_LDBL cannot be used as initializers
     * in arrays.  The above init'ed with zero, so need to patch here.
     */
    {
  /* -NAN_Q */
  #if (defined LMS_NAN_Q_IS_ZERO)
      int i;
      FP q_nan;
      q_nan = ZERO_q;
      q_nan /= q_nan;
      for( i=0; i<=9; i++ ){
	ASSIGN2( vd[i].res, vd[i].arg, q_nan );
      }
      ASSIGN2( vd[1].arg, vd[6].arg, -q_nan );
      ASSIGN2( vd[1].res, vd[6].res, -q_nan );
      vd[10].res = q_nan;
      vd[11].res = q_nan;
  #endif /* -NAN_Q */
    }

  #ifdef TEST_SNAN
    /*
     * If and when Signaling NaNs are supported (LIA-1 and IEEE-754 together)
     * the first 5 vectors are reserved for setting them up.
     */
    {
      unsigned int ui;
      snan_q( &vd[0].arg, "" );
      snan_q( &vd[1].arg, "-" );
      snan_q( &vd[2].arg, "2" );
      snan_q( &vd[3].arg, "3" );
      snan_q( &vd[4].arg, "4" );
      for( ui = 0u; ui < 5u; ui++ ){
	vd[ui].res = vd[ui].arg * RX; /* Quiet form of Signaling Nan */
      }
    }
  #endif

  #ifndef FIX_FUNCS_nan
    ASSIGN2( vd[7].arg, vd[7].res, TRIGGER nan_q("7") );
    ASSIGN2( vd[8].arg, vd[8].res, TRIGGER nan_q("8") );
    ASSIGN2( vd[9].arg, vd[9].res, TRIGGER nan_q("9") );
  #endif

    /*
     * In case compiler converts INF to Pseudo-inf or NaN
     * In case complier will not allow INF_FLT/DBL/LDBL initializers.
     */
    {
      FP inf_q;
      inf_q = ZERO_q;
      inf_q = ONE_q/inf_q;
  /* -INF_Q */
  #if (defined LMS_INF_Q_IS_ZERO)
      vd[11].arg = -inf_q;
  #endif /* -INF_Q */

  /* INF_Q */
  #if (defined LMS_INF_Q_IS_ZERO)
      vd[10].arg = inf_q;
  #else
      ck( !isnan_q(vd[10].arg) && isinf_q(vd[10].arg) );
  #endif /* INF_Q */
    }

    /*
     * In case compiler flushes denormals to zero.
     * In case compiler converts denormals to wrong value.
     * In case compiler converts MIN_* to wrong value.
     */
    {
      VOLATILE FP minp1_q = min_q * onep1_q;
  #if (!defined FIX_FLUSH_RUN_DEN_BOTH)
      VOLATILE FP minm1_q = min_q * (ONE_q-EPS_q);

    /* DEN_Q */
    #if (defined LMS_DEN_Q_IS_ZERO)
      #ifndef FIX_FLUSH_RUN_DEN_BOTH
      vd[13].arg = den_q;
      vd[13].res = den_q;
      #endif
    #else
      ck( vd[13].arg != ZERO_q );
    #endif /* DEN_Q */

    /* MINm1_Q */
    #if (defined LMS_MINm1_Q_IS_ZERO)
      #ifndef FIX_FLUSH_RUN_DEN_BOTH
      vd[14].arg = minm1_q;
      vd[14].res = minm1_q;
      #endif
    #else
      ck( vd[14].arg == minm1_q );
    #endif /* MINm1_Q */

  #endif

  /* MINp1_Q */
  #if (defined LMS_MINp1_Q_IS_ZERO)
      ASSIGN2( vd[16].arg, vd[16].res, minp1_q );
  #endif /* MINp1_Q */

  /* MIN_Q */
  #if (defined LMS_MIN_Q_IS_ZERO)
      ASSIGN2( vd[15].arg, vd[15].res, min_q );
  #endif /* MIN_Q */
    }
    {
  /* -ZERO_q */
  #if (defined LMS_mZERO)
  #endif /* -ZERO_q */
    }

#endif

  }else{ /* non IEEE_VECTORS */

#ifndef TESTING_IEEE_VECTORS
    {
  /* MIN_Q */
  #if (defined LMS_MIN_Q_IS_ZERO) || (defined LMS_USING_FP_EXP)
      ASSIGN2( vd[1].arg, vd[1].res, (min_q+min_q) );
  #endif /* MIN_Q */
    }

  /* FP_EXP() */
  #if (defined LMS_USING_FP_EXP)
    #if 2 == FP_RADIX
    if(need_fixes_3_28){
      FP ulps = FPK(32.);
      unsigned int ui;
      for( ui=28u; 3u<=ui; ui-- ){
	vd[ui].arg = ONE_q / ulps;
	ulps *= TWO_q;
      }
    }
    #endif
  #endif /* FP_EXP() */
#endif

  }
} /* cleanup */

/*================================================================*/
static int bypass(vector vd[], const unsigned int ui){
  int skip = 0;

#if (defined FIX_F_OF_NAN_IS_NAN) || (defined FIX_NAN_sin)
  if( isnan_q(vd[ui].arg) ) skip = 1;
#endif

#ifdef FIX_ZERO_sin
  if( !isnan_q(vd[ui].arg) && (ZERO_q == vd[ui].arg) ) skip = 1;
#endif

#ifdef FIX_DEN_sin
  if( !isnan_q(vd[ui].arg) && (ZERO_q != vd[ui].arg)
     && (-min_q < vd[ui].arg) && (vd[ui].arg < min_q) ) skip = 1;
#endif

#ifdef FIX_SMALL_sin
  if( !isnan_q(vd[ui].arg) && (ZERO_q != vd[ui].arg)
     && (-EPS_q <= vd[ui].arg) && (vd[ui].arg <= EPS_q) ) skip = 1;
#endif

#ifdef FIX_LARGE_sin
  if( !isnan_q(vd[ui].arg)  && isfinite_q(vd[ui].arg)
	&& ((vd[ui].arg <= -MAX_q/RX) || (MAX_q/RX <= vd[ui].arg))
      ) skip = 1;
#endif

#if (defined FIX_INF_sin) || ((defined FIX_BAD_FAKE_DECFP) && (10 == FP_RADIX))
  if( !isnan_q(vd[ui].arg) && !isfinite_q(vd[ui].arg) ) skip = 1;
#endif

#ifdef FIX_NAN_RES_sin
  if( isnan_q(vd[ui].res) ) skip = 1;
#endif

#ifdef FIX_ZERO_RES_sin
  if( !isnan_q(vd[ui].res) && (ZERO_q == vd[ui].res) ) skip = 1;
#endif

#ifdef FIX_DEN_RES_sin
  if( !isnan_q(vd[ui].res) && (ZERO_q != vd[ui].res)
     && (-min_q < vd[ui].res) && (vd[ui].res < min_q) ) skip = 1;
#endif

#ifdef FIX_SMALL_RES_sin
  if( !isnan_q(vd[ui].res) && (ZERO_q != vd[ui].res)
     && (-EPS_q <= vd[ui].res) && (vd[ui].res <= EPS_q) ) skip = 1;
#endif

#ifdef FIX_LARGE_RES_sin
  if( !isnan_q(vd[ui].res)  && isfinite_q(vd[ui].res)
	&& ((vd[ui].res <= -MAX_q/RX) || (MAX_q/RX <= vd[ui].res))
      ) skip = 1;
#endif

#ifdef FIX_INF_RES_sin
  if( !isnan_q(vd[ui].res) && !isfinite_q(vd[ui].res) ) skip = 1;
#endif

  return skip;
} /* bypass */

/*================================================================*/
static const FP ulp_fun = ERR_f;	/* Perhaps, 0.5 */
static FP INT_ERR_f = ERR_f;
static FP res1;				/*  FUT(  arg[i] ) w/ flags clear */
static FP ulps;				/* ulp_diff_q( FUT(x), f(x) )	*/
static FP max_ulp_err;
static int flags1;			/* flags after FUT */
static int errno1;			/* errno after FUT */
static modes mode0;			/* rounding, precision, traps enable */
static unsigned int     true_rnd0;	/* actual rounding modes before FUT */
static unsigned int     true_rnd1;	/* actual rounding modes after FUT */

/*================================================================*/
/*
 *  Make sure FUT works for .5*ulp <= val < 1*ulp, with val having more and more digits.
 */
static void chk_near_one( void ){
#if (!defined FIX_FUNCS_sin)
  {
    FP lo = EPS_q * RXm1 / RX;			/* either 1./2. or 9./10. */
    FP val = ZERO_q;
    int digits;

    for(digits=1; digits<=PREC_q; digits++){
      val += lo;			/* append digit on low end */
      true_rnd0 = get_true_rnd();
      CLEAR_ALL;			/* clear IEEE exceptions */
      res1 = FUT( val );
      flags1 = fetestexcept( FE_IEEE_EXCEPT ); /* flags from FUT */
  #ifdef LMS_MODES_q
      SET_RND(R_DEF);			/* set rounding direction or goto death */
  #else
      true_rnd1 = get_true_rnd();
      ck( true_rnd1 == true_rnd0 );
      if( ckFailed ){
	(void)printf(" True rounding: after(%#x) != before(%#x)\n",
		     true_rnd1, true_rnd0);
      }
  #endif
  #if (!defined FIX_FE_FLAGS) && (!defined LMS_FLAG_q) && (!defined FIX_INEXACT_sin)
      ck( FE_INEXACT == flags1 );	/* just inexact */
  #endif
      ulps = ulp_diff_q(res1, val) ;
      if( max_ulp_err < ulps ) max_ulp_err = ulps;
      if( ONE_q <= ERR_f ){
	ck( ulps <= INT_ERR_f );
      }else{
	ck( val == res1 );		/* is it identity */
      }
      if( ckFailed ){
	(void)printf("Digits=%i\n", digits);
	PRINT_VAL(" Argument", val);
	PRINT_VAL("Ulp error", ulps);
	PRINT_VAL(" Expected", val );
	PRINT_VAL(" Computed", res1);
      }
      lo /= RX;				/* shift right one radix position */
    }/* digits */
  }
death:;
#endif
} /* chk_near_one */

/*================================================================*/
static int chk_res1( vector vd[], unsigned int ui ){
  int rc = 1;				/* assume will fail */
#if (!defined FIX_FUNCS_sin)
  modes mode1;			/* rounding, precision, traps enable */
  {
    mode0 = fegetmodes();		/* control modes */
    true_rnd0 = get_true_rnd();

    CLEAR_ALL;				/* clear IEEE exceptions */
    errno = 0;
    res1 = FUT( vd[ui].arg );		/* FUT - funct under test */
    errno1 = errno;			/* errno from FUT */
    flags1 = fetestexcept( FE_IEEE_EXCEPT ); /* flags from FUT */

  #ifdef LMS_MODES_q
    SET_RND(R_DEF);			/* set rounding direction or goto death */
  #else
    true_rnd1 = get_true_rnd();
    mode1 = fegetmodes();		/* control modes */
    ck( (mode1 == mode0) && (true_rnd0 == true_rnd1) );
    if( ckFailed ){
	(void)printf(" Test vector %u: modes: after(%#lx) != before(%#lx)\n",
		     ui, mode1, mode0 );
	(void)printf(" True rounding: after(%#x) != before(%#x)\n",
		     true_rnd1, true_rnd0);
	PRINT_ARGS;
    }
  #endif

  #ifdef FIX_NAN_GOT_sin
    if( isnan_q(res1) ) goto death;
  #endif

  #ifdef FIX_ZERO_GOT_sin
    if( ZERO_q == res1 ) goto death;
  #endif

  #ifdef FIX_ARG_GOT_sin
    if( vd[ui].arg == res1 ) goto death;
  #endif

  #if (defined FIX_INEXACT_sin)
    vd[ui].flags_req &= ~FE_INEXACT;
    vd[ui].flags_opt &= ~FE_INEXACT;
    flags1 &= ~FE_INEXACT;
  #else
    chk_rnd( vd[ui].arg, vd[ui].ulp, vd, ui, __LINE__ ); /* See if directed rounding honored */
  #endif

    if( memcmp( &res1, &vd[ui].res, SIZ_FP ) != 0 ){
	if( BIT4BIT == vd[ui].ulp ){	/* expected exact match */
  #if (defined IMPL_COHORT_ZERO) && (10 == FP_RADIX)
	  ck( (res1==vd[ui].res) && (signbit(res1)==signbit(vd[ui].res)) );
  #elif ((2==LD_IEEE) && (2==FP_RADIX) && (SIZ_LDBL==SIZ_FP))
	  ck( dbldblcmp( &res1, &vd[ui].res ) == 0 );
  #else
	  ck( memcmp( &res1, &vd[ui].res, SIZ_FP ) == 0 );
  #endif
	  if( ckFailed ){
	    (void)printf(" Test vector %u: FUT not exact match\n", ui);
	    PRINT_ARGS;
	    PRINT_VAL(" Expected ", vd[ui].res);
	    PRINT_VAL(" Computed ", res1);
	  }
	}else if( SIGN_NAN == vd[ui].ulp ){	/* expected same NaN (w/o signs) */
	    ck( nosigncmp_q( res1, vd[ui].res ) );
	    if( ckFailed ){
		(void)printf(" Test vector %u: FUT not 'same' NaN\n", ui);
		PRINT_ARGS;
		PRINT_VAL(" Expected ", vd[ui].res);
		PRINT_VAL(" Computed ", res1);
	    }
	}else if( SOME_NAN == vd[ui].ulp ){	/* expected some NaN */
	    ck( isnan_q( res1 ) );
	    if( ckFailed ){
		(void)printf(" Test vector %u: FUT not a NaN\n", ui);
		PRINT_ARGS;
		PRINT_VAL(" Expected ", vd[ui].res);
		PRINT_VAL(" Computed ", res1);
	    }
	}else if( isnan_q(res1) ){	/* unexpected NaN */
	    ck( !isnan_q( res1 ) );
	    {
		(void)printf(" Test vector %u: FUT is a NaN\n", ui);
		PRINT_ARGS;
		PRINT_VAL(" Expected ", vd[ui].res);
		PRINT_VAL(" Computed ", res1);
	    }
	}else if( SOME_ZERO == vd[ui].ulp ){	/* expected some zero */
	    ck( ZERO_q == res1 );
	    if( ckFailed ){
		(void)printf(" Test vector %u: FUT not a zero\n", ui);
		PRINT_ARGS;
		PRINT_VAL(" Expected ", vd[ui].res);
		PRINT_VAL(" Computed ", res1);
	    }
	}else{				/* expected close match */
	    ulps = ulp_diff_q(res1, vd[ui].res);
	    if( max_ulp_err < ulps ) max_ulp_err = ulps;
	    ck( ulps <= vd[ui].ulp );	/* either INT_ERR_f or ULP_LARGE_ARG_f */
	    if( ckFailed ){
		(void)printf(" Test vector %u: FUT not close enough: ", ui);
		PRINT_VAL("Ulp error ", ulps);
		PRINT_ARGS;
		PRINT_VAL(" Expected ", vd[ui].res);
		PRINT_VAL(" Computed ", res1);
	    }
	}
    }

  #ifndef FIX_RANGE_sin
    if( !isnan_q( res1 ) ){
	FP range_lo = -ONE_q;
	FP range_hi = +ONE_q;
	ck( (range_lo <= res1) && (res1 <= range_hi) );
	if( ckFailed ){
	    (void)printf(" Test vector %u: FUT outside range\n", ui);
	    PRINT_ARGS;
	    PRINT_VAL(" Computed ", res1);
	    PRINT_VAL(" Range lo ", range_lo);
	    PRINT_VAL(" Range hi ", range_hi);
	}
    }
  #endif

  #if (!defined FIX_FE_FLAGS) && (!defined LMS_FLAG_q)
    ck( (flags1 & vd[ui].flags_req) == vd[ui].flags_req );
    if( ckFailed ){
	(void)printf(" Test vector %u: flags: got(%#x) != required(%#x)\n",
			ui, (unsigned)flags1, (unsigned)vd[ui].flags_req );
	PRINT_ARGS;
    }

    ck( (flags1 & ~(vd[ui].flags_req | vd[ui].flags_opt)) == 0 );
    if( ckFailed ){
	(void)printf(" Test vector %u: flags: got(%#x) more than expected(%#x)\n",
			ui, (unsigned)flags1,
			(unsigned)(vd[ui].flags_req | vd[ui].flags_opt) );
	PRINT_ARGS;
    }
  #endif

    /*
     * Are errno and the flags consistent with each other?
     */
  #if (!defined FIX_ERRNO_MATH) && (!defined FIX_ERRNO_MATH_sin)
    if( chk_errno_flags( errno1, flags1, vd, ui ) ){
      (void)printf(" errno & flags inconsistent in g1150000 at line %i\n", __LINE__);
      PRINT_ARGS;
    }
  #endif
  }

  rc = 0;				/* indicate no problems */

death:;
#endif
  return rc;
} /* chk_res1 */

/*================================================================*/
static void chk_res2( vector vd[], unsigned int ui ){
#if (!defined FIX_FUNCS_sin)
  FP res2;			/*  FUT(  arg[i] ) w/ flags set   */
  int flags2;			/* flags after FUT */
  int errno2;			/* errno after FUT */
  modes mode2;			/* rounding, precision, traps enable */
  unsigned int     true_rnd2;	/* actual rounding modes		*/
  {
    RAISE_ALL;				/* set IEEE exceptions */
    errno = 0;
    res2 = FUT( vd[ui].arg );		/* FUT - funct under test */
    errno2 = errno;			/* errno from FUT */
    flags2 = fetestexcept( FE_ALL_EXCEPT ); /* flags from FUT */

  #ifdef LMS_MODES_q
    SET_RND(R_DEF);			/* set rounding direction or goto death */
  #else
    true_rnd2 = get_true_rnd();
    mode2 = fegetmodes();		/* control modes */
    ck( (mode2 == mode0) && (true_rnd0 == true_rnd2) );
    if( ckFailed ){
	(void)printf(" Test vector %u: modes: after(%#lx) != before(%#lx)\n",
		     ui, mode2, mode0 );
	(void)printf(" True rounding: after(%#x) != before(%#x)\n",
		     true_rnd2, true_rnd0);
	PRINT_ARGS;
    }
  #endif

  #if (!defined FIX_INEXACT_sin)
    raise_flags = 1;
    chk_rnd( vd[ui].arg, vd[ui].ulp, vd, ui, __LINE__ ); /* See if directed rounding honored */
    raise_flags = 0;
  #endif

    ck( errno2 == errno1 );
    if( ckFailed ){
	(void)printf(" Test vector %u: errno1=%i, errno2=%i\n",
			ui, (int)errno1, (int)errno2 );
	PRINT_ARGS;
    }

  #if (!defined FIX_FE_FLAGS) && (!defined FIX_FLAGS_feraiseexcept) && (!defined FIX_CLR_FLAGS_sin)
    ck( flags2 == FE_ALL_EXCEPT );
    if( ckFailed ){
	(void)printf(" Test vector %u: FUT cleared some flags\n", ui);
	PRINT_ARGS;
	(void)printf(" Flags got(%#x) not equal expected(%#x)\n",
		(unsigned)flags2, (unsigned)FE_ALL_EXCEPT );
    }
  #endif

  #ifndef FIX_DEPENDS_FLAGS_sin
    ck( memcmp( &res1, &res2, SIZ_FP ) == 0 );
    if( ckFailed ){
	(void)printf(" Test vector %u: FUT depends upon flags\n", ui);
	PRINT_ARGS;
	PRINT_VAL(" Flags clear>", res1);
	PRINT_VAL(" Flags set  >", res2);
    }
  #endif
  }
death:;
#endif
} /* chk_res2 */

/*================================================================*/
static void chk_res3( vector vd[], unsigned int ui ){
#if (!defined FIX_FUNCS_sin)
  FP res3;			/* -FUT( -arg[i] ) w/ flags clear */
  int flags3;			/* flags after FUT */
  int errno3;			/* errno after FUT */
  modes mode3;			/* rounding, precision, traps enable */
  unsigned int     true_rnd3;	/* actual rounding modes		*/
  {
    {
      CLEAR_ALL;			/* clear IEEE exceptions */
      errno = 0;
      res3 = -FUT( -vd[ui].arg );	/* FUT - funct under test */
      errno3 = errno;
      flags3 = fetestexcept( FE_IEEE_EXCEPT ); /* flags from FUT */

  #ifdef LMS_MODES_q
      SET_RND(R_DEF);			/* set rounding direction or goto death */
  #else
      true_rnd3 = get_true_rnd();
      mode3 = fegetmodes();		/* control modes */
      ck( (mode3 == mode0) && (true_rnd0 == true_rnd3) );
      if( ckFailed ){
	(void)printf(" Test vector %u: modes: after(%#lx) != before(%#lx)\n",
		     ui, mode3, mode0 );
	(void)printf(" True rounding: after(%#x) != before(%#x)\n",
		     true_rnd3, true_rnd0);
	PRINT_ARGS;
      }
  #endif

  #if (defined FIX_INEXACT_sin)
      flags3 &= ~FE_INEXACT;
  #else
      chk_rnd( -vd[ui].arg, vd[ui].ulp, vd, ui, __LINE__ ); /* See if directed rounding honored */
  #endif

  #if (!defined FIX_FE_FLAGS) && (!defined LMS_FLAG_q)
      {
	ck( (flags1 & vd[ui].flags_req) == (flags3 & vd[ui].flags_req) );
	if( ckFailed ){
	    (void)printf(" Test vector %u: flags: FUT(x) != -FUT(-x)\n", ui);
	    PRINT_ARGS;
	    (void)printf(" Flags FUT(x)(%#x) not equal -FUT(-x)(%#x)\n",
			(unsigned)(flags1 & vd[ui].flags_req),
			(unsigned)(flags3 & vd[ui].flags_req) );
	}
      }
  #endif

      if( !isnan_q(vd[ui].res)
  #ifdef LMS_SIGNED_ZERO_BUGS
	&& (ZERO_q != res3)
  #endif
	&& ( memcmp( &res1, &res3, SIZ_FP ) != 0 )
      ){
  #if (defined FIX_EVENODD_sin)
	ulps = ulp_diff_q(res3, res1);
	if( max_ulp_err < ulps ) max_ulp_err = ulps;
	ck(!isnan_q(ulps) && (ulps <= INT_ERR_f));
  #elif ((2==LD_IEEE) && (2==FP_RADIX) && (SIZ_LDBL==SIZ_FP))
	ck( dbldblcmp( &res1, &res3 ) == 0 );
  #else
	ck( memcmp( &res1, &res3, SIZ_FP ) == 0 );
  #endif
	if( ckFailed ){
	    (void)printf(" Test vector %u: FUT(x) != -FUT(-x): ", ui);
  #if (defined FIX_EVENODD_sin)
	    PRINT_VAL("Ulp error ", ulps);
  #endif
	    PRINT_ARGS;
	    PRINT_VAL("  FUT( x) ", res1);
	    PRINT_VAL(" -FUT(-x) ", res3);
	}
      }

      /*
       * Are errno and the flags consistent with each other?
       */
  #if (!defined FIX_ERRNO_MATH) && (!defined FIX_ERRNO_MATH_sin)
      if( chk_errno_flags( errno3, flags3, vd, ui ) ){
	(void)printf(" errno & flags inconsistent in g1150000 at line %i\n", __LINE__);
	PRINT_ARGS;
      }
  #endif
    }
  }
death:;
#endif
} /* chk_res3 */

/*================================================================*/
/*
 * Make sure type generic matches type specific
 */
static void chk_generic( vector vd[], unsigned int ui ){
#if (!defined FIX_FUNCS_sin)
  #if (!defined FIX_OVERLOAD) && (!defined FIX_OVERLOAD_sin) \
   && (!defined FIX_OVERLOAD10) && (!defined FIX_OVERLOAD10_sin)
  {
    {
      L_FP specific;
      L_FP generic;
      true_rnd0 = get_true_rnd();
      CLEAR_ALL;			/* clear IEEE exceptions */
      specific = (SPECIFIC)( vd[ui].arg );	/* type specific function */
      generic =         FUT( vd[ui].arg );	/* FUT - funct under test */
    #ifdef LMS_MODES_q
      SET_RND(R_DEF);			/* set default rounding direction, or death */
    #else
      true_rnd1 = get_true_rnd();
      ck( true_rnd1 == true_rnd0 );
      if( ckFailed ){
	(void)printf(" True rounding: after(%#x) != before(%#x)\n",
		     true_rnd1, true_rnd0);
      }
    #endif
      if( isnan(generic) ){
	ck( isnan(specific) );
	if( ckFailed ){
	  PRINT_ARGS;
	  PRINT_VALL("  type generic ",generic);
	  PRINT_VALL("  type specific",specific);
	}
      }else{
	ck( generic == specific );
	if( ckFailed ){
	  PRINT_ARGS;
	  PRINT_VALL("  type generic ",generic);
	  PRINT_VALL("  type specific",specific);
	}
      }
    }
  }
death:;
  #endif
#endif
} /* chk_generic */

/*================================================================*/
/*
 * If FUT(min+ulp) raises spurious underflow, try to find largest
 * argument where bad behaviour starts.
 */
static void chk_spur_under( void ){
#if (!defined FIX_FUNCS_sin)
  #ifndef FIX_BAD_UF_GOT_sin
  FP mid = min_q*(onep1_q);
  CLEAR_ALL;			/* clear IEEE exceptions */
  res1 = FUT(mid);
  flags1 = fetestexcept( FE_IEEE_EXCEPT ); /* flags from FUT */
  if( flags1 & FE_UNDERFLOW ){
    FP hi = ONE_q;
    FP lo = mid;
    int cycles;

    ck( 0 == flags1 );
    for( cycles=0; cycles < 9876; cycles++ ){
      mid = lo + (hi-lo)*FPK(0.5);
      if( mid == lo ) break;	/* converged, so exit loop		*/
      CLEAR_ALL;		/* clear IEEE exceptions */
      res1 = FUT( mid );
      flags1 = fetestexcept( FE_IEEE_EXCEPT ); /* flags from FUT */
      if( flags1 & FE_UNDERFLOW ){
      	lo = mid;
      }else{
      	hi = mid;
      }
    }/* cycles */
    PRINT_VAL("FUT(x) appears to raise spurious underflow at\n x < ", hi);
  }
  #endif
#endif
} /* chk_spur_under */

/*================================================================*/
/*
 * If FUT(max) is a NaN, try to find smallest argument where bad
 * behaviour starts.
 */
static void chk_when_nan( void ){
#if (!defined FIX_FUNCS_sin)
  #ifndef FIX_NAN_GOT_sin
  FP mid = MAX_q;
  res1 = FUT(mid);
  if( isnan_q(res1) ){
    FP hi = mid;
    FP lo = ZERO_q;
    int cycles;

    ck( !isnan_q(res1) );
    PRINT_VAL("  mid", mid);
    for( cycles=0; cycles < 9876; cycles++ ){
      mid = lo + (hi-lo)*FPK(0.5);
      if( mid == lo ) break;	/* converged, so exit loop		*/
      res1 = FUT( mid );
      if( isnan_q( res1 ) ){
      	hi = mid;
      }else{
      	lo = mid;
      }
    }/* cycles */
    PRINT_VAL("FUT(x) appears to be NaN at\n x > ", lo);
    PRINT_VAL("   lo", lo );
    PRINT_VAL("  mid", mid);
    PRINT_VAL("   hi", hi );
  }
  #endif
#endif
} /* chk_when_nan */

/*================================================================*/
static void doit(void){		/* Internal block */
#if (!defined FIX_FUNCS_sin)

  #if (!defined FIX_STATIC_VECTORS)
  static
  #endif
  vector vd[] = {
  #include VECTORS		/* The test vectors */
  };

  unsigned int ui;		/* array index */

  if( ulp_fun < (FP)100000uL ){
      unsigned long int uli;
  #if (defined FIX_ULI_FROM_LARGE_FP_CONST)
      uli = (unsigned long int)INT_ERR_f;	/* floor(ERR_f) */
  #else
      uli = (unsigned long int)ulp_fun;		/* floor(ERR_f) */
  #endif
      INT_ERR_f = (FP)uli;
      if( INT_ERR_f < ulp_fun ) INT_ERR_f++;	/* ceil(ERR_f) */
      ck( (ulp_fun <= INT_ERR_f) && (INT_ERR_f < (ulp_fun+ONE_q)) );
      if( ckFailed ){
	  PRINT_VAL("  ulp_fun",   ulp_fun);
	  PRINT_VAL("    ERR_f",     ERR_f);
	  PRINT_VAL("INT_ERR_f", INT_ERR_f);
      }
  }
  max_ulp_err = ZERO_q;

  ck( (ZERO_q < ERR_f) && "ULP_sin needs to be greater than zero; try 0.5");
  if( ckFailed ) return;

  #ifdef TESTING_IEEE_VECTORS
    #define WHICH 1
  #else
    #define WHICH 0
  #endif
  #ifdef LMS_NEED_FIXES_3_28
    #define LMS_3_28 1
  #else
    #define LMS_3_28 0
  #endif
  cleanup(vd, WHICH, LMS_3_28);

  #ifdef     OVERRIDE_TV
    #include OVERRIDE_TV
  #endif

  chk_near_one();			/* Check with args near one */

  for( ui = 0u; ui < SIZE(vd); ui++ ){	/* Check with test vectors */
    printed_arg = 0;

  #ifdef LIMIT_DOMAIN
    {
      D_FP d = (D_FP)vd[ui].arg;	/* drop bottom bits */
      vd[ui].arg = (FP)d;
      if( !isnan_q(vd[ui].arg) ){	/*  and add inexact */
	vd[ui].flags_opt |= FE_INEXACT;
	if( !vd[ui].arg ) vd[ui].res = vd[ui].arg;/* 0 => 0 */
      }else{
	vd[ui].res = (FP)d;
      }
    }
  #endif

    if( bypass( vd, ui ) ) continue;

    if( isfinite_q(vd[ui].arg) ){
	if( LARGE_ARG_f < vd[ui].arg){
	    vd[ui].ulp = ULP_LARGE_ARG_f;
	}else if(ulp_fun == vd[ui].ulp){
	    vd[ui].ulp = INT_ERR_f;
	}
    }

    if( chk_res1( vd, ui ) ) continue;
    chk_res2( vd, ui );

    if( !isnan_q(vd[ui].arg) ){
      chk_res3( vd, ui );
    }

    chk_generic( vd, ui );

    if( NumFail >> 6 ) break;		/* too many failures */
  }/* ui loop */

  chk_spur_under();			/* look for spurious underflow */
  chk_when_nan();			/* look for NaN from large arg */

death:;
  if( NumFail ){
    PRINT_VAL("max_ulp_err should be", max_ulp_err);
  }

#endif

  if( PRTDEBUG(NumFail) ){
    (void)printf("\n... Symbols defined ...\n");
#ifdef DEBUG
    (void)printf("DEBUG\n");
#endif
#ifdef DEBUG2
    (void)printf("DEBUG2\n");
#endif
#ifdef DEBUG3
    (void)printf("DEBUG3\n");
#endif
#ifdef FIX_DFP_FLAGS
    (void)printf("FIX_DFP_FLAGS\n");
#endif
#ifdef FIX_FE_FLAGS
    (void)printf("FIX_FE_FLAGS\n");
#endif
#ifdef FIX_FUNCS10_nan
    (void)printf("FIX_FUNCS10_nan\n");
#endif
#ifdef FIX_FUNCS10_sin
    (void)printf("FIX_FUNCS10_sin\n");
#endif
#ifdef FIX_FUNCS_nan
    (void)printf("FIX_FUNCS_nan\n");
#endif
#ifdef FIX_FUNCS_sin
    (void)printf("FIX_FUNCS_sin\n");
#endif
#ifdef FIX_OVERLOAD
    (void)printf("FIX_OVERLOAD\n");
#endif
#ifdef FIX_OVERLOAD10
    (void)printf("FIX_OVERLOAD10\n");
#endif
#ifdef FIX_OVERLOAD10_sin
    (void)printf("FIX_OVERLOAD10_sin\n");
#endif
#ifdef FIX_OVERLOAD_sin
    (void)printf("FIX_OVERLOAD_sin\n");
#endif
#ifdef FIX_STATIC_VECTORS
    (void)printf("FIX_STATIC_VECTORS\n");
#endif
#ifdef FIX_ULI_FROM_LARGE_FP_CONST
    (void)printf("FIX_ULI_FROM_LARGE_FP_CONST\n");
#endif
#ifdef FIX_VOLATILE
    (void)printf("FIX_VOLATILE\n");
#endif
#ifdef LIMIT_DOMAIN
    (void)printf("LIMIT_DOMAIN\n");
#endif
#ifdef LMS_DEN_Q_IS_ZERO
    (void)printf("LMS_DEN_Q_IS_ZERO\n");
#endif
#ifdef LMS_INF_Q_IS_ZERO
    (void)printf("LMS_INF_Q_IS_ZERO\n");
#endif
#ifdef LMS_MINm1_Q_IS_ZERO
    (void)printf("LMS_MINm1_Q_IS_ZERO\n");
#endif
#ifdef LMS_MINp1_Q_IS_ZERO
    (void)printf("LMS_MINp1_Q_IS_ZERO\n");
#endif
#ifdef LMS_MIN_Q_IS_ZERO
    (void)printf("LMS_MIN_Q_IS_ZERO\n");
#endif
#ifdef LMS_mZERO
    (void)printf("LMS_mZERO\n");
#endif
#ifdef LMS_NAN_Q_IS_ZERO
    (void)printf("LMS_NAN_Q_IS_ZERO\n");
#endif
#ifdef LMS_NEED_FIXES_3_28
    (void)printf("LMS_NEED_FIXES_3_28\n");
#endif
#ifdef LMS_ONEm1_Q_IS_ZERO
    (void)printf("LMS_ONEm1_Q_IS_ZERO\n");
#endif
#ifdef LMS_ONEp1_Q_IS_ZERO
    (void)printf("LMS_ONEp1_Q_IS_ZERO\n");
#endif
#ifdef LMS_SIGNED_ZERO_BUGS
    (void)printf("LMS_SIGNED_ZERO_BUGS\n");
#endif
#ifdef LMS_USING_FP_EXP
    (void)printf("LMS_USING_FP_EXP\n");
#endif
#ifdef NEED_VOLATILE
    (void)printf("NEED_VOLATILE\n");
#endif
#ifdef IMPL_COHORT_ZERO
    (void)printf("IMPL_COHORT_ZERO\n");
#endif
#ifdef IMPL_INEXACT10_sin
    (void)printf("IMPL_INEXACT10_sin\n");
#endif
#ifdef IMPL_NO_ERRNO_RND10_sin
    (void)printf("IMPL_NO_ERRNO_RND10_sin\n");
#endif
#ifdef IMPL_NO_ERRNO_RND_sin
    (void)printf("IMPL_NO_ERRNO_RND_sin\n");
#endif
#ifdef IMPL_NO_STRONG_RND10_sin
    (void)printf("IMPL_NO_STRONG_RND10_sin\n");
#endif
#ifdef IMPL_NO_STRONG_RND_sin
    (void)printf("IMPL_NO_STRONG_RND_sin\n");
#endif
#ifdef IMPL_NO_WEAK_RND10_sin
    (void)printf("IMPL_NO_WEAK_RND10_sin\n");
#endif
#ifdef IMPL_NO_WEAK_RND_sin
    (void)printf("IMPL_NO_WEAK_RND_sin\n");
#endif
#ifdef IMPL_UF_WEIRD10_sin
    (void)printf("IMPL_UF_WEIRD10_sin\n");
#endif
#ifdef IMPL_UF_WEIRD_sin
    (void)printf("IMPL_UF_WEIRD_sin\n");
#endif
#ifdef FIX_BAD_FAKE_DECFP
    (void)printf("FIX_BAD_FAKE_DECFP\n");
#endif
#ifdef DEN_FLAGS
    (void)printf("DEN_FLAGS\n");
#endif
#ifdef FIX_ARG_GOT10_sin
    (void)printf("FIX_ARG_GOT10_sin\n");
#endif
#ifdef FIX_ARG_GOT_sin
    (void)printf("FIX_ARG_GOT_sin\n");
#endif
#ifdef FIX_BAD_UF_GOT10_sin
    (void)printf("FIX_BAD_UF_GOT10_sin\n");
#endif
#ifdef FIX_BAD_UF_GOT_sin
    (void)printf("FIX_BAD_UF_GOT_sin\n");
#endif
#ifdef FIX_CLOSE_MIN_DBL
    (void)printf("FIX_CLOSE_MIN_DBL\n");
#endif
#ifdef FIX_CLOSE_MIN_FLT
    (void)printf("FIX_CLOSE_MIN_FLT\n");
#endif
#ifdef FIX_CLOSE_MIN_LDBL
    (void)printf("FIX_CLOSE_MIN_LDBL\n");
#endif
#ifdef FIX_CLR_FLAGS10_sin
    (void)printf("FIX_CLR_FLAGS10_sin\n");
#endif
#ifdef FIX_CLR_FLAGS_sin
    (void)printf("FIX_CLR_FLAGS_sin\n");
#endif
#ifdef FIX_DEN10_sin
    (void)printf("FIX_DEN10_sin\n");
#endif
#ifdef FIX_DEN_RES10_sin
    (void)printf("FIX_DEN_RES10_sin\n");
#endif
#ifdef FIX_DEN_RES_sin
    (void)printf("FIX_DEN_RES_sin\n");
#endif
#ifdef FIX_DEN_sin
    (void)printf("FIX_DEN_sin\n");
#endif
#ifdef FIX_DEPENDS_FLAGS10_sin
    (void)printf("FIX_DEPENDS_FLAGS10_sin\n");
#endif
#ifdef FIX_DEPENDS_FLAGS_sin
    (void)printf("FIX_DEPENDS_FLAGS_sin\n");
#endif
#ifdef FIX_ERRNO_MATH
    (void)printf("FIX_ERRNO_MATH\n");
#endif
#ifdef FIX_ERRNO_MATH10_sin
    (void)printf("FIX_ERRNO_MATH10_sin\n");
#endif
#ifdef FIX_ERRNO_MATH_sin
    (void)printf("FIX_ERRNO_MATH_sin\n");
#endif
#ifdef FIX_EVENODD10_sin
    (void)printf("FIX_EVENODD10_sin\n");
#endif
#ifdef FIX_EVENODD_sin
    (void)printf("FIX_EVENODD_sin\n");
#endif
#ifdef FIX_FLAG10_sin
    (void)printf("FIX_FLAG10_sin\n");
#endif
#ifdef FIX_FLAGS_feraiseexcept
    (void)printf("FIX_FLAGS_feraiseexcept\n");
#endif
#ifdef FIX_FLAG_sin
    (void)printf("FIX_FLAG_sin\n");
#endif
#ifdef FIX_FLUSH_RUN_DEN_BOTH
    (void)printf("FIX_FLUSH_RUN_DEN_BOTH\n");
#endif
#ifdef FIX_F_OF_NAN_IS_NAN
    (void)printf("FIX_F_OF_NAN_IS_NAN\n");
#endif
#ifdef FIX_INEXACT10_sin
    (void)printf("FIX_INEXACT10_sin\n");
#endif
#ifdef FIX_INEXACT_sin
    (void)printf("FIX_INEXACT_sin\n");
#endif
#ifdef FIX_INF10_sin
    (void)printf("FIX_INF10_sin\n");
#endif
#ifdef FIX_INF_RES10_sin
    (void)printf("FIX_INF_RES10_sin\n");
#endif
#ifdef FIX_INF_RES_sin
    (void)printf("FIX_INF_RES_sin\n");
#endif
#ifdef FIX_INF_sin
    (void)printf("FIX_INF_sin\n");
#endif
#ifdef FIX_LARGE10_sin
    (void)printf("FIX_LARGE10_sin\n");
#endif
#ifdef FIX_LARGE_RES10_sin
    (void)printf("FIX_LARGE_RES10_sin\n");
#endif
#ifdef FIX_LARGE_RES_sin
    (void)printf("FIX_LARGE_RES_sin\n");
#endif
#ifdef FIX_LARGE_sin
    (void)printf("FIX_LARGE_sin\n");
#endif
#ifdef FIX_MODES10_sin
    (void)printf("FIX_MODES10_sin\n");
#endif
#ifdef FIX_MODES_sin
    (void)printf("FIX_MODES_sin\n");
#endif
#ifdef FIX_NAN10_sin
    (void)printf("FIX_NAN10_sin\n");
#endif
#ifdef FIX_NAN_GOT10_sin
    (void)printf("FIX_NAN_GOT10_sin\n");
#endif
#ifdef FIX_NAN_GOT_sin
    (void)printf("FIX_NAN_GOT_sin\n");
#endif
#ifdef FIX_NAN_RES10_sin
    (void)printf("FIX_NAN_RES10_sin\n");
#endif
#ifdef FIX_NAN_RES_sin
    (void)printf("FIX_NAN_RES_sin\n");
#endif
#ifdef FIX_NAN_sin
    (void)printf("FIX_NAN_sin\n");
#endif
#ifdef FIX_RANGE10_sin
    (void)printf("FIX_RANGE10_sin\n");
#endif
#ifdef FIX_RANGE_sin
    (void)printf("FIX_RANGE_sin\n");
#endif
#ifdef FIX_SMALL10_sin
    (void)printf("FIX_SMALL10_sin\n");
#endif
#ifdef FIX_SMALL_RES10_sin
    (void)printf("FIX_SMALL_RES10_sin\n");
#endif
#ifdef FIX_SMALL_RES_sin
    (void)printf("FIX_SMALL_RES_sin\n");
#endif
#ifdef FIX_SMALL_sin
    (void)printf("FIX_SMALL_sin\n");
#endif
#ifdef FIX_ZERO10_sin
    (void)printf("FIX_ZERO10_sin\n");
#endif
#ifdef FIX_ZERO_GOT_sin
    (void)printf("FIX_ZERO_GOT_sin\n");
#endif
#ifdef FIX_ZERO_RES10_sin
    (void)printf("FIX_ZERO_RES10_sin\n");
#endif
#ifdef FIX_ZERO_RES_sin
    (void)printf("FIX_ZERO_RES_sin\n");
#endif
#ifdef FIX_ZERO_sin
    (void)printf("FIX_ZERO_sin\n");
#endif
#ifdef LARGE_ARG_f
    (void)printf("LARGE_ARG_f\n");
#endif
#ifdef NAN_FLAGS
    (void)printf("NAN_FLAGS\n");
#endif
#ifdef ERR_M0
    (void)printf("ERR_M0\n");
#endif
#ifdef ERR_NAN
    (void)printf("ERR_NAN\n");
#endif
#ifdef DFP_TEST_LVL
    (void)printf("DFP_TEST_LVL is %li\n", (long int) DFP_TEST_LVL );
#endif
#ifdef FPCE_TEST_LVL
    (void)printf("FPCE_TEST_LVL is %li\n", (long int) FPCE_TEST_LVL );
#endif
#ifdef ULP_sin
    (void)printf("ULP_sin is %g\n", (double) ULP_sin );
#endif
#define  str(s) # s
#define xstr(s) str(s)
#ifdef ulp_diff_r
    (void)printf("ulp_diff is %s is %s\n",  str(ulp_diff_r), xstr(ulp_diff_r(0,0)));
#endif
    (void)printf("FUT is %s is %s\n",  str(FUT), xstr(FUT(0)));
#ifndef FIX_xFPK
    (void)printf("ERR_f is %s is %s\n",  str(ERR_f), xstr(ERR_f));
#endif
  }

} /* Internal block */
